==> arte7-0.1.0.tar.gz: arte7-0.1.0/arte7.py <==
#!/use/bin/env python
# -*- coding: utf-8 -*-

"""A simple command line tool for interacting with the arte+7 database.

arte is a great TV station and they even provide most of their programs
for 7 days after the initial airing.  But their website is truly horrible.
So this program might be useful for anyone who wants to enjoy arte
without using their website.
"""

from __future__ import print_function

import argparse
import codecs
import json
import multiprocessing
import sys
from textwrap import fill

from filecachetools import ttl_cache

try:
	from urllib2 import urlopen
	import BaseHTTPServer
except ImportError:
	from urllib.request import urlopen
	import http.server as BaseHTTPServer

try:
	import colorama
except ImportError:
	colorama = None


@ttl_cache('arte7', ttl=60*60*24*7)
def get_unicode(url):
	"""Wrapper around urlopen() that returns a unicode string."""
	fh = urlopen(url)
	# hacky, but works ok
	encoding = fh.headers.get('content-type').split('=')[-1]
	string = fh.read()
	fh.close()
	return string.decode(encoding)


def get_single(vid, lang):
	"""Return description, meta and stream_url for a single video."""
	lang_map = {
		'de': 'D',
		'fr': 'F',
	}
	l = lang_map[lang]
	url = ('http://arte.tv/papi/tvguide/videos/stream/player/'
		'%s/%s_PLUS7-%s/ALL/ALL.json' % (l, vid, l))
	data = json.loads(get_unicode(url))
	data = data['videoJsonPlayer']

	desc = data.get('VDE')
	meta = data.get('infoProg')
	try:
		stream_url = data['VSR']['HTTP_REACH_EQ_1']['url']
	except KeyError:
		stream_url = data['VSR']['HTTP_MP4_EQ_1']['url']

	return desc, meta, stream_url


def _get_single(args):
	"""Wrapper used with multiprocessing.Pool.map()."""
	return get_single(*args)


def _filter(video, q):
	"""Return true if video matches the search query q."""
	def match(hay, needle):
		if hay is None:
			return False
		else:
			return hay.lower().find(needle.lower()) >= 0

	return match(video.get('title'), q) or match(video.get('desc'), q)


def get_all(lang, q=''):
	url = 'http://www.arte.tv/guide/%s/plus7.json' % lang
	data = json.loads(get_unicode(url))

	candidates = [v for v in data['videos'] if _filter(v, q)]
	args = [(v['em'], lang) for v in candidates]

	pool = multiprocessing.Pool()
	responses = pool.map(_get_single, args)

	videos = []
	for video, response in zip(candidates, responses):
		v = {
			'title': video.get('title'),
			'airdate': video.get('airdate_long'),
			'image_url': video.get('image_url'),
		}
		v['desc'], v['meta'], v['stream_url'] = response
		videos.append(v)
	return videos


def format_console(videos, colors=True):
	"""Format `videos` for displaying in the console."""
	ret = []
	for video in videos:
		if colors and colorama is not None:
			tpl = (
				colorama.Fore.GREEN + "%(title)s\n" +
				colorama.Fore.YELLOW + "%(airdate)s - %(meta)s\n" +
				colorama.Fore.BLUE + "%(stream_url)s\n\n" +
				colorama.Style.RESET_ALL + fill(video["desc"]) + "\n\n")
		else:
			tpl = (
				"%(title)s\n" +
				"%(airdate)s - %(meta)s\n" +
				"%(stream_url)s\n\n" +
				fill(video["desc"]) + "\n\n")
		ret.append(tpl % video)
	return '\n'.join(ret)


def format_html(videos):
	"""Format `videos` as  HTML."""
	css = """
		.videos {
			list-style: none;
			margin: 0;
			padding: 0;
		}
		.video {
			border: 1px solid silver;
			padding: 1em;
			margin: 2em 0;
			overflow: hidden;  /* clearfix */
		}
		.video img {
			float: left;
			width: 30%;
			margin-right: 1em;
		}
		/* colors */
		a:link {
			color: #f9471b;
		}
		html {
			background-color: #202020;
			color: white;
			font-family: ProximaNova, Arial;
			padding: 1em;
		}
		body {
			background-color: transparent;
			max-width: 70em;
			margin: 0 auto;
		}
		.video {
			background-color: #ecebe9;
			color: #282828;
		}
	"""
	html_tpl = """<!DOCTYPE html>
		<html>
			<head>
				<meta charset="utf8" />
				<title>ARTE+7</title>
				<style>%(css)s</style>
			</head>
			<body>
				<h1>Arte+7</h1>
				<ul class="videos">%(list)s</ul>
			</body>
		</html>"""
	item_tpl = """<li class="video">
			<h2><a href="%(stream_url)s">%(title)s</a></h2>
			<img src="%(image_url)s" alt="" />
			<p>%(desc)s</p>
			<span class="airdata">%(airdate)s</span>
			-
			<span class="meta">%(meta)s</span>
		</li>"""
	return html_tpl % {
		'list': '\n'.join((item_tpl % v for v in videos)),
		'css': css,
	}


class HTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
	"""HTTP Request Handler showing a single HTML page."""

	html = ''

	def do_GET(self):
		self.send_response(200)
		self.send_header("Content-type", 'text/html')
		self.end_headers()
		self.wfile.write(self.html.encode('utf8'))


def start_server(port):
	"""Start HTTP Server with HTTPRequestHandler on port `port`."""
	server_address = ('localhost', port)
	httpd = BaseHTTPServer.HTTPServer(server_address, HTTPRequestHandler)
	sa = httpd.socket.getsockname()
	print("Serving HTTP on", sa[0], "port", sa[1], "...")
	httpd.serve_forever()


def parse_args():
	"""Parse command line arguments."""
	parser = argparse.ArgumentParser(description=__doc__)
	parser.add_argument('-l', '--language', choices=['de', 'fr'], default='de')
	parser.add_argument('-t', '--to',
		choices=['console', 'html', 'serve', 'json'],
		default='console', help="output format")
	parser.add_argument('-b', '--boring', action='store_true',
		help="no colors in console output")
	parser.add_argument('-p', '--port', type=int, help="port used for serve",
		default=8000)
	parser.add_argument('q', nargs='?', help="search term", default='')
	return parser.parse_args()


def main():
	"""Entry point for the command line tool."""
	UTF8Writer = codecs.getwriter('utf8')
	sys.stdout = UTF8Writer(sys.stdout)

	args = parse_args()
	videos = get_all(args.language, q=args.q)

	if args.to == 'html':
		print(format_html(videos))
	elif args.to == 'serve':
		HTTPRequestHandler.html = format_html(videos)
		start_server(args.port)
	elif args.to == 'json':
		print(json.dumps(videos, indent=2))
	else:
		colors = not args.boring and sys.stdout.isatty()
		print(format_console(videos, colors=colors))


if __name__ == '__main__':
	main()

==> arte7-1.0.0.tar.gz: arte7-1.0.0/arte7.py <==
#!/use/bin/env python
# -*- coding: utf-8 -*-

"""A simple command line tool for interacting with the arte+7 database.

arte is a great TV station and they even provide most of their programs
for 7 days after the initial airing.  But their website is truly horrible.
So this program might be useful for anyone who wants to enjoy arte
without using their website.
"""

from __future__ import print_function

import argparse
import codecs
import json
import multiprocessing
import sys
from textwrap import fill

from filecachetools import ttl_cache

try:
	from urllib2 import urlopen
	import BaseHTTPServer
except ImportError:
	from urllib.request import urlopen
	import http.server as BaseHTTPServer

try:
	import colorama
except ImportError:
	colorama = None

__version__ = '1.0.0'


@ttl_cache('arte7', ttl=60*60*24*7)
def get_unicode(url):
	"""Wrapper around urlopen() that returns a unicode string."""
	fh = urlopen(url)
	# hacky, but works ok
	encoding = fh.headers.get('content-type').split('=')[-1]
	string = fh.read()
	fh.close()
	return string.decode(encoding)


def get_single(url):
	"""Return description, meta and stream_url for a single video."""
	data = json.loads(get_unicode(url))
	data = data['video']

	desc = data.get('VDE')
	meta = data.get('infoProg')
	for vsr in data['VSR']:
		if vsr['VFO'] == 'HBBTV' and vsr['VQU'] == 'SQ':
			stream_url = vsr['VUR']

	return desc, meta, stream_url


def _get_single(args):
	"""Wrapper used with multiprocessing.Pool.map()."""
	return get_single(*args)


def _filter(video, q):
	"""Return true if video matches the search query q."""
	def match(hay, needle):
		if hay is None:
			return False
		else:
			return hay.lower().find(needle.lower()) >= 0

	return match(video.get('TIT'), q) or match(video.get('DTW'), q)


def get_all(lang, q='', limit=1000):
	lang_map = {
		'de': 'D',
		'fr': 'F',
	}
	url = ('http://arte.tv/papi/tvguide/videos/plus7/program/%s/L3/ALL'
		'/ALL/-1/AIRDATE_DESC/%i/0/DE_FR.json') % (lang_map[lang], limit)
	data = json.loads(get_unicode(url))

	key = 'program%sList' % lang.upper()
	candidates = [v for v in data[key] if _filter(v, q)]
	args = [(v['VDO']['videoStreamUrl'],) for v in candidates]

	pool = multiprocessing.Pool()
	# HACK to get KeyboardInterrupt to work.
	# See https://stackoverflow.com/questions/1408356
	responses = pool.map_async(_get_single, args).get(99999)

	videos = []
	for video, response in zip(candidates, responses):
		v = {
			'title': video.get('TIT'),
			'airdate': video['VDO'].get('VDA'),
			'image_url': video['IMG'].get('IUR'),
		}
		v['desc'], v['meta'], v['stream_url'] = response
		v['filename'] = v['title'] + '.' + v['stream_url'].rsplit('.', 1)[-1]
		videos.append(v)
	return videos


def format_console(videos, colors=True):
	"""Format `videos` for displaying in the console."""
	ret = []
	for video in videos:
		if colors and colorama is not None:
			tpl = (
				colorama.Fore.GREEN + "%(title)s\n" +
				colorama.Fore.YELLOW + "%(airdate)s - %(meta)s\n" +
				colorama.Fore.BLUE + "%(stream_url)s\n\n" +
				colorama.Style.RESET_ALL + fill(video["desc"] or "") + "\n\n")
		else:
			tpl = (
				"%(title)s\n" +
				"%(airdate)s - %(meta)s\n" +
				"%(stream_url)s\n\n" +
				fill(video["desc"] or "") + "\n\n")
		ret.append(tpl % video)
	return '\n'.join(ret)


def format_html(videos):
	"""Format `videos` as  HTML."""
	css = """
		.videos {
			list-style: none;
			margin: 0;
			padding: 0;
		}
		.video {
			border: 1px solid silver;
			padding: 1em;
			margin: 2em 0;
			overflow: hidden;  /* clearfix */
		}
		.video img {
			float: left;
			width: 30%;
			margin-right: 1em;
		}
		/* colors */
		a:link {
			color: #f9471b;
		}
		html {
			background-color: #202020;
			color: white;
			font-family: ProximaNova, Arial;
			padding: 1em;
		}
		body {
			background-color: transparent;
			max-width: 70em;
			margin: 0 auto;
		}
		.video {
			background-color: #ecebe9;
			color: #282828;
		}
	"""
	html_tpl = """<!DOCTYPE html>
		<html>
			<head>
				<meta charset="utf8" />
				<title>ARTE+7</title>
				<style>%(css)s</style>
			</head>
			<body>
				<h1>Arte+7</h1>
				<ul class="videos">%(list)s</ul>
			</body>
		</html>"""
	item_tpl = """<li class="video">
			<h2><a href="%(stream_url)s" download="%(filename)s">%(title)s</a></h2>
			<img src="%(image_url)s" alt="" />
			<p>%(desc)s</p>
			<span class="airdata">%(airdate)s</span>
			-
			<span class="meta">%(meta)s</span>
		</li>"""
	return html_tpl % {
		'list': '\n'.join((item_tpl % v for v in videos)),
		'css': css,
	}


class HTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
	"""HTTP Request Handler showing a single HTML page."""

	html = ''

	def do_GET(self):
		self.send_response(200)
		self.send_header("Content-type", 'text/html')
		self.end_headers()
		self.wfile.write(self.html.encode('utf8'))


def start_server(port):
	"""Start HTTP Server with HTTPRequestHandler on port `port`."""
	server_address = ('localhost', port)
	httpd = BaseHTTPServer.HTTPServer(server_address, HTTPRequestHandler)
	sa = httpd.socket.getsockname()
	print("Serving HTTP on", sa[0], "port", sa[1], "...")
	httpd.serve_forever()


def parse_args():
	"""Parse command line arguments."""
	parser = argparse.ArgumentParser(description=__doc__)
	parser.add_argument('--version', '-V', action='version', version=__version__)
	parser.add_argument('-l', '--language', choices=['de', 'fr'], default='de')
	parser.add_argument('-t', '--to',
		choices=['console', 'html', 'serve', 'json'],
		default='console', help="output format")
	parser.add_argument('-b', '--boring', action='store_true',
		help="no colors in console output")
	parser.add_argument('-p', '--port', type=int, help="port used for serve",
		default=8000)
	parser.add_argument('q', nargs='?', help="search term", default='')
	return parser.parse_args()


def main():
	"""Entry point for the command line tool."""
	UTF8Writer = codecs.getwriter('utf8')
	sys.stdout = UTF8Writer(sys.stdout)

	args = parse_args()
	videos = get_all(args.language, q=args.q)

	if args.to == 'html':
		print(format_html(videos))
	elif args.to == 'serve':
		HTTPRequestHandler.html = format_html(videos)
		start_server(args.port)
	elif args.to == 'json':
		print(json.dumps(videos, indent=2))
	else:
		colors = not args.boring and sys.stdout.isatty()
		print(format_console(videos, colors=colors))


if __name__ == '__main__':
	main()
